﻿'use strict';

var chatDashboard = {};

$(function () {
    function displayActiveTab() {
        $('a[href="#tab_a"]').tab('show');
    }

    var viewModel;
    function subscribeViewModel(newViewModel) {
        viewModel = newViewModel;

        var sessionIdToSelect = null;
        viewModel.selectedSessionId.extend({ notify: 'always' });
        viewModel.selectedSessionId.subscribe(function (newSessionId) {
            if (!document.hidden && newSessionId) {
                setTimeout(function () {
                    let $messageTextBox = $('#msgInput_' + newSessionId);
                    if ($messageTextBox.length > 0) {
                        //if (viewModel.selectedChatItem().session().WaitingMessage !== null) { //pouze ten klient, ktery chat vybral
                        displayActiveTab();
                        $messageTextBox.focus();
                        //}
                    }
                    else
                        sessionIdToSelect = newSessionId;
                }, 0);
            }
        });

        var lastWaitingChatItemsLength = 0;
        viewModel.waitingChatItems.subscribe(function (newWaitingChatItems) {
            if (newWaitingChatItems.length !== lastWaitingChatItemsLength) {
                if (newWaitingChatItems.length > lastWaitingChatItemsLength)
                    $('#chatQueueTab').effect("pulsate", 1000);

                lastWaitingChatItemsLength = newWaitingChatItems.length;
            }
        });

        viewModel.acceptedByAnybodyChatItems.subscribe(function (newAcceptedByAnybodyChatItems) {
            if (sessionIdToSelect && newAcceptedByAnybodyChatItems.length > 0) {
                setTimeout(function () {
                    let $messageTextBox = $('#msgInput_' + sessionIdToSelect);
                    if ($messageTextBox.length > 0) {
                        displayActiveTab();
                        $messageTextBox.focus();
                        sessionIdToSelect = null;
                    }
                }, 0);
            }
        });

        viewModel.guestHistory.detail.subscribe(function (newDetail) {
            $("#dialog-guestHistory").closest("div[role='dialog']").find(".ui-dialog-title").text(newDetail ? Localization_Chat_ChatsHistory + " - " + moment.utc(newDetail.CreationDateUtc).local().format("l LT") : null);
        });
    }

    function initGuestHistoryDialog() {
        $("#dialog-guestHistory").dialog({
            autoOpen: false,
            resizable: true,
            width: "auto",
            height: "600",
            modal: false,
            buttons: [{
                text: Localization_Close,
                click: function() {
                    $(this).dialog("close");
                }
            }],
            close: function (e) {
                viewModel.clearHistorySummary();
            }
        });
    }

    function initSelectOperatorDialog() {
        $("#dialog-selectOperator").dialog({
            autoOpen: false,
            resizable: false,
            width: "auto",
            height: "auto",
            modal: true,
            buttons: [{
                    text: Localization_Select,
                    click: function () {
                        let selectedSession = viewModel.selectedChatItem() !== null ? viewModel.selectedChatItem().session : null;
                        let selectedOperator = viewModel.selectedOperator();

                        if (selectedSession !== null && selectedOperator !== null) {
                            viewModel.addSelectedOperator();
                            $(this).dialog("close");
                        }
                    }
                },
                {
                    text: Localization_Close,
                    click: function () {
                        $(this).dialog("close");
                    }
                }
            ],
            close: function (e) {
                viewModel.clearOperatorsOfChatProfile();
            }
        });
    }

    function initRefuseConfirmationDialog() {
        $("#dialog-refuseConfirmation").dialog({
            autoOpen: false,
            resizable: false,
            width: "auto",
            height: "auto",
            modal: true
        });
    }

    $('#btRefuseSession').click(function () {
        viewModel.refuseSession(viewModel.selectedChatItem().session(), true, null);
        $("#dialog-refuseConfirmation").dialog('close');
    });

    $('#btRefuseSessionAndExistingTicket').click(function () {
        openChatTicketRelationDialog();
    });

    function openChatTicketRelationDialog() {
        $("#ChatSessionTicket").val([]).trigger('change');
        $("#dialog-chatTicketRelation").html("<div class='loading-32'></div>");
        $("#dialog-chatTicketRelation").dialog("open");
        $("#dialog-chatTicketRelation").load(RQ_URL + "/Chat/ChatTicketRelation/" + viewModel.selectedChatItem().sessionId, function () {
            $("#dialog-chatTicketRelation").dialog("close");
            $("#dialog-chatTicketRelation").dialog("open");
        });
    }

    function initChatTicketRelationDialog() {
        $("#dialog-chatTicketRelation").dialog({
            autoOpen: false,
            resizable: false,
            width: "900",
            height: "auto",
            modal: true,
            buttons: [
                {
                    text: Localization_Save,
                    click: function () {
                        var selectedChatItem = viewModel.selectedChatItem();
                        if (selectedChatItem) {
                            selectedChatItem.createTicketRelationsAndRefuse($("#ChatSessionTicket").val()).then(function () {
                                $("#dialog-chatTicketRelation").dialog("close");
                                $("#dialog-refuseConfirmation").dialog('close');
                            }).catch(function (error) {
                                console.error(error);
                            });
                        }
                    }
                },
                {
                    text: Localization_Close,
                    click: function () {
                        $(this).dialog("close");
                    }
                }
            ]
        });
    }

    $('.aRefuseSessionAndNewTicket').click(function () {
        var ticketType = $(this).data('ticket-type');
        var session = viewModel.selectedChatItem().session();
        viewModel.refuseSession(session, true, null);
        $("#dialog-refuseConfirmation").dialog('close');
        window.location.href = RQ_URL + '/Tickets/New/' + ticketType + '?chat=' + session.SessionId;
    });

    function openGuestHistoryDetailDialog() {
        $('a[href="#guestHistoryDetail"]').tab('show');
        $("#dialog-guestHistory").dialog('open');
    }

    function openGuestHistorySummaryDialog() {
        $('a[href="#guestHistorySummary"]').tab('show');
        $("#dialog-guestHistory").closest("div[role='dialog']").find(".ui-dialog-title").text(Localization_Chat_ChatsHistory + " - " + viewModel.selectedChatItem().guestUserSession().User.FullName);
        $("#dialog-guestHistory").dialog('open');
    }

    function openSelectOperatorDialog() {
        viewModel.getUsers();
        viewModel.loadOperatorsForSelectedSession();
        $("#dialog-selectOperator").dialog('open');
    }

    function openRefuseConfirmationDialog() {
        $("#dialog-refuseConfirmation").dialog('open');
    }

    function operatorTemplate(data) {
        if (typeof data.id === 'string' && data.id.length > 0)
        {
            var operatorData = JSON.parse(data.text);
            return $('<div></div>', {
                "class": "vertical-center-container",
                html: [
                $('<div></div>', {
                    "class": "color-indication",
                    css: { "background": operatorData.canReceiveNewGuests ? "green" : "red", "margin": "2px;" }
                }),
                $('<span></span>', {
                    css: { "padding": "0px 6px" },
                    text: operatorData.fullName
                })]
            });
        }

        return "";
    }

    initSelectOperatorDialog();
    initGuestHistoryDialog();
    initRefuseConfirmationDialog();
    initChatTicketRelationDialog();
    chatDashboard.displayActiveTab = displayActiveTab;
    chatDashboard.operatorTemplate = operatorTemplate;
    chatDashboard.subscribeViewModel = subscribeViewModel;
    chatDashboard.openSelectOperatorDialog = openSelectOperatorDialog;
    chatDashboard.openGuestHistorySummaryDialog = openGuestHistorySummaryDialog;
    chatDashboard.openGuestHistoryDetailDialog = openGuestHistoryDetailDialog;
    chatDashboard.openRefuseConfirmationDialog = openRefuseConfirmationDialog;
});