﻿'use strict';

var chatMainboard = {};

$(function () {
    var viewModel;
    var MIN_DATE_TICKS = -8640000000000000;
    var lastLatestAlertDate = new Date(MIN_DATE_TICKS);

    $('#chatActiveUsers .btn-group > button[data-toggle="dropdown"]').click(function () {
        if ($('#chatActiveUsers .btn-group > div.dropdown-menu').is(":visible")) {
            viewModel.getUsers();
        }
    });

    function subscribeViewModel(newViewModel) {
        viewModel = newViewModel;
        if (!newViewModel.operatorMode) {
            return;
        }

        function messageAdded(message, chatItem) {
            if (chatItem instanceof ChatGuestItem) {
                if (message.Type !== chatUtils.ChatMessageTypeEnum.COMMON_INCOMPLETE && message.UserProviderKey !== newViewModel.userProviderKey && newViewModel.getMyAttitude(chatItem.session()) === true) {
                    let chatProfile = newViewModel.getChatProfileFromMessage(message);
                    if (chatProfile) {
                        if (chatProfile.WebNotifications && chatProfile.WebNotifications.MessageReceived && message.UserProviderKey !== newViewModel.userProviderKey) {
                            displayChatMessageWebNotification(message, false, message.FullName && message.FromOperator ? message.FullName : chatItem.guestUserSession().User.FullName);
                        }

                        if (chatProfile.NewMessageSound) {
                            var soundElement = document.getElementById("chatSound" + chatProfile.NewMessageSound);
                            if (soundElement !== null) {
                                if (!soundElement.paused)
                                    soundElement.currentTime = 0;

                                soundElement.volume = chatProfile.NewMessageSoundVolume / 100;
                                soundElement.play();
                            }
                        }
                    }
                }
            }
        }

        function subscribeChatItemMessages(chatItem) {
            return chatItem.viewModel().messages.subscribe(function (changes) {
                changes.forEach(function (change) {
                    if (change.status === 'added') {
                        messageAdded(change.value, chatItem);
                    }
                });
            }, null, "arrayChange");
        }

        var chatItemsSubscriptions = {};
        newViewModel.chatItems.subscribe(function (changes) {
            changes.forEach(function (change) {
                let chatItem = change.value;
                if (chatItem instanceof ChatGuestItem) {
                    if (change.status === 'added') {
                        if (chatItem.viewModel()) {
                            chatItemsSubscriptions[chatItem.sessionId] = subscribeChatItemMessages(chatItem);
                        } else {
                            chatItemsSubscriptions[chatItem.sessionId] = chatItem.viewModel.subscribe(function (newSessionViewModel) {
                                if (newSessionViewModel) {
                                    chatItemsSubscriptions[chatItem.sessionId].dispose();
                                    chatItemsSubscriptions[chatItem.sessionId] = subscribeChatItemMessages(chatItem);
                                }
                            });
                        }
                    }
                    else if (change.status === 'deleted') {
                        if (chatItemsSubscriptions[chatItem.sessionId]) {
                            chatItemsSubscriptions[chatItem.sessionId].dispose();
                            delete chatItemsSubscriptions[chatItem.sessionId];
                        }
                        else
                            console.error(chatItemsSubscriptions);
                    }
                }
            });

        }, null, "arrayChange");

        viewModel.acceptedUnspottedOrMyWaitingChatItems.subscribe(function (newChatItems) {
            if (!viewModel.chatProfiles || !viewModel.canReceiveNewGuests())
                return;

            let latestAlertDate = new Date(MIN_DATE_TICKS);
            let latestChatItem;

            $(newChatItems).each(function (index, chatItem) {
                if (chatItem.waitingDate() && chatItem.waitingDate() >= latestAlertDate)
                {
                    latestAlertDate = chatItem.waitingDate();
                    latestChatItem = chatItem;
                }
            });

            if (lastLatestAlertDate < latestAlertDate)
            {
                lastLatestAlertDate = latestAlertDate;
                var chatProfile = latestChatItem.chatProfile;
                if (chatProfile) {
                    if (chatUtils.chatItemIsBeyondChatProfileLimit(latestChatItem, viewModel.chatItemsByChatProfile()[chatProfile.Key]))
                        return;

                    if (chatProfile.WebNotifications && chatProfile.WebNotifications.NewChatCreated)
                        displayChatMessageWebNotification(latestChatItem.session().WaitingMessage, true);

                    if (chatProfile.NewChatSound) {
                        var soundElement = document.getElementById("chatSound" + chatProfile.NewChatSound);
                        if (soundElement !== null) {
                            if (!soundElement.paused)
                                soundElement.currentTime = 0;

                            soundElement.volume = chatProfile.NewChatSoundVolume / 100;
                            soundElement.play();
                        }
                    }
                }
            }
        });
    }

    function displayChatMessageWebNotification(chatMessage, newChat, userFullName) {
        function html2text(html) {
            var tag = document.createElement('div');
            tag.innerHTML = html;
            return tag.innerText;
        }

        if ("Notification" in window && Notification.permission === "granted") {
            var hidden;
            if (typeof document.hidden !== "undefined") { // Opera 12.10 and Firefox 18 and later support 
                hidden = "hidden";
            } else if (typeof document.msHidden !== "undefined") {
                hidden = "msHidden";
            } else if (typeof document.webkitHidden !== "undefined") {
                hidden = "webkitHidden";
            }

            if (hidden !== undefined && (chatMessage.Type === chatUtils.ChatMessageTypeEnum.COMMON ||
                chatMessage.Type === chatUtils.ChatMessageTypeEnum.ATTACHMENTS ||
                chatMessage.Type === chatUtils.ChatMessageTypeEnum.START_CALL)) {
                var faviconLink = document.querySelector("link[rel*='icon']");
                var notification = new Notification(newChat ? Localization_Chat_NewChat : "{0} {1}\u2026".format(userFullName, Localization_Chat_Wrote), {
                    body: chatMessage.Type === chatUtils.ChatMessageTypeEnum.START_CALL ? Localization_Chat_Call : html2text(chatMessage.Text),
                    icon: faviconLink.href,
                    badge: faviconLink.href,
                    requireInteraction: true,
                    timestamp: new Date(chatMessage.Date),
                    tag: chatMessage.SessionId,
                    renotify: true
                });

                notification.onclick = function (event) {
                    notification.close();
                    displayChatSession(chatMessage.SessionId, newChat);
                };
            }
        }
    }

    function getSubscribedViewModel() {
        return viewModel;
    }

    function displayChatSession(sessionId, accept)
    {
        window.focus();
        if (!window.chatDashboard) {
            var redirectedUrl = RQ_URL + "/Chat?SessionId=" + sessionId;
            if (accept) {
                viewModel.acceptSession(sessionId, false, function () {
                    window.location.replace(redirectedUrl);
                });
            }
            else
                window.location.replace(redirectedUrl);
        }
        else {
            if (accept)
                viewModel.acceptSession(sessionId, false, null);
            else
                viewModel.selectChatItem(sessionId, true, false);
        }
    }

    function displayAndCreateChatSession(guestProviderKey, chatProfileKey) {
        window.focus();
        if (!window.chatDashboard) {
            return viewModel.createGuestUserSessionAndAcceptIfNeeded(guestProviderKey, chatProfileKey).then(function (sessionId) {
                if (sessionId) {
                    var redirectedUrl = RQ_URL + "/Chat?SessionId=" + sessionId;
                    window.location.replace(redirectedUrl);
                }
                return sessionId;
            });
        }
        else {
            return viewModel.createGuestUserSessionAndAcceptIfNeeded(guestProviderKey, chatProfileKey);
        }
    }

    var LAST_UNSPOTTED_MESSAGE_RECENT_TRESHOLD = 5000;
    function lastUnspottedMessageIsRecent(chatItem) {
        let viewModel = chatItem.viewModel();
        if (viewModel) {
            let lastUnspottedMessage = viewModel.lastUnspottedMessage();
            if (lastUnspottedMessage) {
                return chatItem.currentDate() - new Date(lastUnspottedMessage.Date) <= LAST_UNSPOTTED_MESSAGE_RECENT_TRESHOLD;
            }
        }

        return false;
    }

    //ready
    chatMainboard.displayChatSession = displayChatSession;
    chatMainboard.displayAndCreateChatSession = displayAndCreateChatSession;
    chatMainboard.subscribeViewModel = subscribeViewModel;
    chatMainboard.getSubscribedViewModel = getSubscribedViewModel;
    chatMainboard.lastUnspottedMessageIsRecent = lastUnspottedMessageIsRecent;
});